from datetime import datetime
from typing import List, Union


class Statistic:
    def __init__(self, timestamp: str, value: int):
        self.timestamp: datetime = datetime.fromtimestamp(int(timestamp) / 1000)
        self.value: int = value

    def __repr__(self):
        return f"Statistic({self.timestamp}, {self.value})"


class DashboardStatistics:
    def __init__(self, data: dict):
        self.events: List[Statistic] = []
        for timestamp, value in data.get("events").items():
            self.events.append(Statistic(timestamp, value))

        self.workers: List[Worker] = []
        for worker in data.get("workerStatistics"):
            self.workers.append(Worker(worker))

        self._data = data

    def __repr__(self):
        return f"DashboardStatistics({self._data})"


class WorkerMetric:
    def __init__(self, metric_id: str, datapoints: Union[dict, float, int]):
        self.metric_id: str = metric_id
        self.statistics: List[Statistic] = []

        if not isinstance(datapoints, dict):
            datapoints = {f"{int(datetime.now().timestamp() * 1000)}": datapoints}
        for timestamp, value in datapoints.items():
            self.statistics.append(Statistic(timestamp, value))


class Worker:
    def __init__(self, data: dict):
        self.id: str = data["id"]
        self.metrics: List[WorkerMetric] = []
        for metric_id, datapoints in data.get("statistics", {}).items():
            self.metrics.append(WorkerMetric(metric_id, datapoints))



import abc
from abc import ABC

from anypoint.http_client.client import HttpClient


class Authentication(ABC):

    def __init__(self):
        self._http_client = None

    @abc.abstractmethod
    def get_token(self, url: str) -> str:
        raise NotImplementedError

    @property
    def http_client(self) -> HttpClient:
        return self._http_client

    @http_client.setter
    def http_client(self, http_client: HttpClient):
        self._http_client = http_client


class Basic(Authentication):

    def __init__(self, username: str, password: str):
        self.username = username
        self.password = password
        super().__init__()

    def get_token(self, url: str) -> str:
        path = f"{url}/accounts/login"
        r = self._http_client.request(path, "POST", body={"username": self.username, "password": self.password})
        return r.get("access_token")


class OAuth2(Authentication):

    def __init__(self, client_id: str, client_secret: str):
        self.client_id = client_id
        self.client_secret = client_secret
        super().__init__()

    def get_token(self, url: str) -> str:
        path = f"{url}/accounts/api/v2/oauth2/token"
        body = {
            "client_id": self.client_id,
            "client_secret": self.client_secret,
            "grant_type": "client_credentials"
        }
        headers = {
            "Content-Type": "application/json"
        }
        r = self._http_client.request(path, "POST", body=body, headers=headers)
        return r.get("access_token")



from __future__ import annotations

import logging
import typing
from typing import List

from anypoint.models.api import Asset

if typing.TYPE_CHECKING:
    from anypoint import Anypoint


class ApiManagerApi:
    def __init__(self, client: Anypoint, log: logging.Logger):
        self._client = client
        self._log = log

    def get_apis(self, organization_id: str, environment_id: str) -> List[Asset]:
        path = f"/apimanager/api/v1/organizations/{organization_id}/environments/{environment_id}/apis"
        assets = self._client.request(path).get("assets", [])
        with open("assets.json", "w") as f:
            import json
            json.dump(assets, f, indent=2)
        return [Asset(asset, self) for asset in assets]



from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from anypoint import Anypoint


class Worker:
    def __init__(self, raw_json, client: "Anypoint"):
        self.id = raw_json.get("id")
        self.host = raw_json.get("host")
        self.port = raw_json.get("port")
        self.status = raw_json.get("status")
        self.deployed_region = raw_json.get("deployedRegion")

        self._data = raw_json
        self._api_client = client

    def __repr__(self):
        return f"Worker({self.id})"


class WorkerStatistic:
    def __init__(self, raw_json, client: "Anypoint"):
        self.id = raw_json.get("id")
        self.disk_read_bytes = self.get_latest_value("diskReadBytes")
        self.disk_write_bytes = self.get_latest_value("diskWriteBytes")
        self.network_in = self.get_latest_value("networkIn")
        self.network_out = self.get_latest_value("networkOut")
        self.memory_total_used = self.get_latest_value("memoryTotalUsed")
        self.memory_percent_used = self.get_latest_value("memoryPercentageUsed")
        self.cpu = self.get_latest_value("cpu")
        self.memory_total_max = raw_json.get("statistics", {})["memoryTotalMax"] if "memoryTotalMax" in raw_json.get(
            "statistics", {}) else -1

        self._data = raw_json
        self._api_client = client

    def get_latest_value(self, metric_name) -> float:
        if metric_name in self._data["statistics"] and self._data["statistics"][metric_name]:
            return list(self._data["statistics"][metric_name].values())[-1]
        return -1

    def __repr__(self):
        return f"Worker {self.id} CPU: {self.cpu}%, MEM: {self.memory_percent_used:.2f}%"



import logging
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from anypoint import Anypoint


class EnvironmentApi:
    def __init__(self, client: "Anypoint", log: logging.Logger):
        self._client = client
        self._log = log



import logging
from typing import Generator, Optional, TYPE_CHECKING

from anypoint.models.environment import Environment
from anypoint.models.organization import Organization

if TYPE_CHECKING:
    from anypoint import Anypoint


class OrganizationApi:
    def __init__(self, client: "Anypoint", log: logging.Logger):
        self._client = client
        self._log = log

    def get_organization(self, org_id: Optional[str] = None) -> Organization:
        if org_id is None:
            data = self._client.me()
            data = data.get("user", {}).get("organization", {})
        else:
            path = f"/accounts/api/organizations/{org_id}"
            data = self._client.request(path)
        return Organization(data, self)

    def get_environments(self, organization_id: str) -> Generator[Environment, None, None]:
        path = f"/accounts/api/organizations/{organization_id}/environments"
        data = self._client.request(path)
        for env in data.get("data", []):
            env["organization_id"] = organization_id
            yield Environment(env, self._client)

    def get_environment_organization(self, environment_id: str) -> Organization:
        path = f"/cloudhub/api/organization"
        headers = {"X-ANYPNT-ENV-ID": environment_id}
        data = self._client.request(path, headers=headers)
        return Organization(data, self)



import logging
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from anypoint import Anypoint


class MonitoringApi:
    def __init__(self, client: "Anypoint", log: logging.Logger):
        self._client = client
        self._log = log
        self._proxy_influx_db: str = ""
        self._influx_db: str = ""

    def sync(self):
        path = "/monitoring/api/sync"
        data = self._client.request(path)
        for p in data.get("proxies", []):
            if p["name"] == "influxdb":
                self._proxy_influx_db = p["id"]
            elif p["name"] == "influx_analytics":
                self._proxy_influx_analytics = p["id"]

    def boot_data(self):
        path = "/monitoring/api/visualizer/api/bootdata"
        data = self._client.request(path)
        influx_data = data.get("Settings", {}).get("datasources", {}).get("influxdb", {})
        if influx_data:
            self._influx_db = influx_data["database"]
            self._proxy_influx_db = influx_data["id"]

    def query(self, query_string: str) -> dict:
        if not self._proxy_influx_db:
            self.boot_data()
        path = f"/monitoring/api/visualizer/api/datasources/proxy/{self._proxy_influx_db}/query"
        return self._client.request(
            path,
            parameters={
                "db": self._influx_db,
                "q": query_string,
                "epoch": "ms"
            }
        )



from __future__ import annotations

from typing import List, TYPE_CHECKING

if TYPE_CHECKING:
    from anypoint.api.api_manager import ApiManagerApi


class Asset:
    def __init__(self, raw_json, client: ApiManagerApi):
        self.id = raw_json.get("id")
        self.org_id = raw_json.get("organizationId")
        self.name = raw_json.get("name")
        self.exchange_asset_name = raw_json.get("exchangeAssetName")
        self.group_id = raw_json.get("groupId")
        self.asset_id = raw_json.get("assetId")
        self.apis: List[Api] = [Api(raw_data, client) for raw_data in raw_json.get("apis", [])]

    def __repr__(self):
        return f"Asset({self.id}, {self.asset_id})"


class Api:
    def __init__(self, raw_json, client: ApiManagerApi):
        self.org_id = raw_json.get("organizationId")
        self.id = raw_json.get("id")
        self.instance_label = raw_json.get("instanceLabel")
        self.group_id = raw_json.get("groupId")
        self.asset_id = raw_json.get("assetId")
        self.asset_version = raw_json.get("assetVersion")
        self.product_version = raw_json.get("productVersion")
        self.description = raw_json.get("description")
        self.tags = raw_json.get("tags")
        self.order = raw_json.get("order")
        self.provider_id = raw_json.get("providerId")
        self.deprecated = raw_json.get("deprecated")
        self.last_active_date = raw_json.get("lastActiveDate")
        self.endpoint_uri = raw_json.get("endpointUri")
        self.environment_id = raw_json.get("environmentId")
        self.is_public = raw_json.get("isPublic")
        self.stage = raw_json.get("stage")
        self.technology = raw_json.get("technology")
        self.pinned = raw_json.get("pinned")
        self.active_contracts_count = raw_json.get("activeContractsCount")
        self.autodiscovery_instance_name = raw_json.get("autodiscoveryInstanceName")

    def __repr__(self):
        return f"Api({self.id}, {self.instance_label})"



from datetime import datetime
from typing import Generator, List, TYPE_CHECKING

from anypoint.models.statistics import DashboardStatistics, Statistic
from anypoint.models.worker import Worker, WorkerStatistic

if TYPE_CHECKING:
    from anypoint.api.application import ApplicationApi


class Application:
    def __init__(self, raw_json, client: "ApplicationApi"):
        self.domain: str = raw_json.get("domain")
        self.full_domain: str = raw_json.get("fullDomain")
        self.id: str = raw_json.get("id")
        self.mule_version: str = raw_json.get("muleVersion")
        self.region: str = raw_json.get("region")
        self.status: str = raw_json.get("status")
        self.filename: str = raw_json.get("filename")
        self.deployment_group_id: str = raw_json.get("deploymentGroupId")

        self.num_workers: int = raw_json.get("workers")
        self.remaining_workers: int = raw_json.get("remainingWorkerCount", 0)
        self.environment_id: str = raw_json.get("environment_id")
        self.statistics = None
        self.worker_statistics: List[WorkerStatistic] = []
        self.workers: List[Worker] = []
        self.worker_type = raw_json.get("workerType")
        self.worker_count = raw_json.get("workers", 0)
        self.file_name = raw_json.get("filename")
        self.href = raw_json.get("href")

        self._data = raw_json
        self._api_client = client

        try:
            self.last_update_time: datetime = datetime.utcfromtimestamp(raw_json.get("lastUpdateTime") / 1000)
        except Exception:
            self.last_update_time: datetime = datetime.now()

    def __repr__(self):
        return f"Application({self.full_domain})"

    def __lt__(self, other):
        return self.domain < other.domain

    def get_statistics(self, date_from: datetime, date_to: datetime) -> Generator[Statistic, None, None]:
        return self._api_client.get_application_statistics(self.environment_id, self.domain, date_from, date_to)

    def get_dashboard_statistics(self, date_from: datetime, date_to: datetime) -> DashboardStatistics:
        return self._api_client.get_dashboard_statistics(self.environment_id,
                                                         self.domain,
                                                         date_from,
                                                         date_to)

    def get_status(self):
        return self._api_client.get_application_status(self.environment_id, self.domain)

    def get_insights(self):
        return self._api_client.get_insights(self.environment_id, self.domain)



from __future__ import annotations

from datetime import datetime
from typing import Optional, TYPE_CHECKING

if TYPE_CHECKING:
    from anypoint import Anypoint


class Destination:
    def __init__(self, raw_json, client: Anypoint, organization_id: str, environment_id: str, region_id: str):
        self.organization_id = organization_id
        self.environment_id = environment_id
        self.region_id = region_id

        self.encrypted: bool = raw_json.get("encrypted")
        self.type: str = raw_json.get("type")
        self.queue_id: str = raw_json.get("queueId")
        self.dead_letter_sources: str = raw_json.get("deadLetterSources")
        self.fifo: bool = raw_json.get("fifo")
        self.default_ttl: int = raw_json.get("defaultTtl")
        self.default_lock_ttl: int = raw_json.get("defaultLockTtl")
        self.default_delivery_delay: int = raw_json.get("defaultDeliveryDelay")
        self.max_deliveries: int = raw_json.get("maxDeliveries")
        self.dead_letter_queue_id: str = raw_json.get("deadLetterQueueId")

        self._data = raw_json
        self._client = client

    def __repr__(self):
        return f"Destination({self.queue_id}, {self.type})"

    def get_queue(self, start_date: Optional[datetime] = None, end_date: Optional[datetime] = None):
        return self._client.mq.get_queue(self.organization_id, self.environment_id, self.region_id, self.queue_id, start_date, end_date)


class Queue:
    def __init__(self, raw_json):
        # {'destination': 'CUSTOMER-CREATE-SFMC-Q', 'messages': [{'date': '2023-03-01T16:34:00.000+00:00', 'value': 144}], 'inflightMessages': [{'date': '2023-03-01T16:34:00.000+00:00', 'value': 0}], 'messagesVisible': [{'date': '2023-03-01T16:34:00.000+00:00', 'value': 144}], 'messagesSent': [{'date': '2023-03-01T16:34:00.000+00:00', 'value': 57}], 'messagesReceived': [{'date': '2023-03-01T16:34:00.000+00:00', 'value': 0}], 'messagesAcked': [{'date': '2023-03-01T16:34:00.000+00:00', 'value': 0}]}
        self.destination: str = raw_json.get("destination")
        # Grab the value of the first item in the list
        self.messages = raw_json.get("messages", [{}])[0].get("value") if raw_json.get("messages") else 0
        self.inflight_messages = raw_json.get("inflightMessages", [{}])[0].get("value") if raw_json.get("inflightMessages") else 0
        self.messages_visible = raw_json.get("messagesVisible", [{}])[0].get("value") if raw_json.get("messagesVisible") else 0
        self.messages_sent = raw_json.get("messagesSent", [{}])[0].get("value") if raw_json.get("messagesSent") else 0
        self.messages_received = raw_json.get("messagesReceived", [{}])[0].get("value") if raw_json.get("messagesReceived") else 0
        self.messages_acked = raw_json.get("messagesAcked", [{}])[0].get("value") if raw_json.get("messagesAcked") else 0

    def __repr__(self):
        return f"Queue({self.destination}, {self.messages=}, {self.inflight_messages=}, {self.messages_visible=}, {self.messages_sent=}, {self.messages_received=}, {self.messages_acked=})"



from typing import Generator, List, TYPE_CHECKING

from anypoint.models.api import Asset
from anypoint.models.application import Application
from anypoint.models.destination import Destination, Queue

if TYPE_CHECKING:
    from anypoint import Anypoint


class Environment:
    def __init__(self, raw_json, client: "Anypoint"):
        self.id: str = raw_json.get("id")
        self.name: str = raw_json.get("name")
        self.is_production: bool = raw_json.get("isProduction")
        self.type: str = raw_json.get("type")
        self.client_id: str = raw_json.get("clientId")
        self.organization_id = raw_json.get("organizationId")
        self.applications: List[Application] = []

        self._data = raw_json
        self._client = client

    def __repr__(self):
        return f"Environment({self.name}, {self.id})"

    def get_applications(self) -> Generator[Application, None, None]:
        return self._client.applications.get_applications(self.id)

    def get_apis(self) -> List[Asset]:
        return self._client.api_manager.get_apis(self.organization_id, self.id)

    def get_organization(self):
        return self._client.organizations.get_environment_organization(self.id)

    def get_mq_queues(self, region_id: str, destinations: List[str] = None):
        return self._client.mq.get_queues(self.organization_id, self.id, region_id, destinations)

    def get_mq_queue(self, region_id: str, destination_id: str) -> Queue:
        return self._client.mq.get_queue(self.organization_id, self.id, region_id, destination_id)

    def get_mq_destinations(self, region_id: str) -> List[Destination]:
        return self._client.mq.get_destinations(self.organization_id, self.id, region_id)



from typing import List, TYPE_CHECKING

from anypoint.models.environment import Environment

if TYPE_CHECKING:
    from anypoint.api.organization import OrganizationApi


class Organization:
    def __init__(self, raw_json, client: "OrganizationApi"):
        self.id: str = raw_json.get("id")
        self.name: str = raw_json.get("name")
        self.owner_id: str = raw_json.get("ownerId")
        self.created_at: str = raw_json.get("createdAt")
        self.updated_at: str = raw_json.get("updatedAt")
        self.is_federated: bool = raw_json.get("isFederated")
        self.org_type: str = raw_json.get("orgType")
        self.domain: str = raw_json.get("domain")
        self.is_root: bool = raw_json.get("isRoot")
        self.is_master: bool = raw_json.get("isMaster")
        self.mfa_required: str = raw_json.get("mfaRequired")
        self.entitlements: "Entitlements" = Entitlements(raw_json.get("entitlements", {}))

        self._data = raw_json
        self._api_client = client

        usage = raw_json.get("usage", {})
        self.standard_connectors = usage.get("standardConnectors", 0)
        self.premium_connectors = usage.get("premiumConnectors", 0)
        self.production_applications = usage.get("productionApplications", 0)
        self.sandbox_applications = usage.get("sandboxApplications", 0)
        self.design_applications = usage.get("designApplications", 0)
        self.production_workers = usage.get("productionWorkers", 0)
        self.sandbox_workers = usage.get("sandboxWorkers", 0)
        self.design_workers = usage.get("designWorkers", 0)
        self.static_ips = usage.get("staticIps", 0)
        self.vpcs = usage.get("vpcs", 0)
        self.vpns = usage.get("vpns", 0)
        self.network_connections = usage.get("networkConnections", 0)
        self.load_balancers = usage.get("loadbalancers", 0)
        self.load_balancer_workers = usage.get("loadbalancerWorkers", 0)
        self.deployment_groups = usage.get("deploymentGroups", 0)

    def get_environments(self) -> List[Environment]:
        return list(self._api_client.get_environments(self.id))


class Entitlements:
    def __init__(self, raw_json: dict):
        self.create_environments: bool = raw_json.get("createEnvironments")
        self.global_deployment: bool = raw_json.get("globalDeployment")
        self.create_sub_orgs: bool = raw_json.get("createSubOrgs")
        self.hybrid: dict = raw_json.get("hybrid", {})
        self.hybrid_insight: bool = raw_json.get("hybridInsight")
        self.hybrid_auto_discover_properties: bool = raw_json.get("hybridAutoDiscoverProperties")
        self.v_cores_production: dict = raw_json.get("vCoresProduction", {})
        self.v_cores_sandbox: dict = raw_json.get("vCoresSandbox", {})
        self.v_cores_design: dict = raw_json.get("vCoresDesign", {})
        self.static_ips: dict = raw_json.get("staticIps", {})
        self.vpcs: dict = raw_json.get("vpcs", {})
        self.vpns: dict = raw_json.get("vpns", {})
        self.network_connections: dict = raw_json.get("networkConnections", {})
        self.worker_logging_override: dict = raw_json.get("workerLoggingOverride", {})
        self.mq_messages: dict = raw_json.get("mqMessages", {})
        self.mq_requests: dict = raw_json.get("mqRequests", {})
        self.object_store_request_units: dict = raw_json.get("objectStoreRequestUnits", {})
        self.object_store_keys: dict = raw_json.get("objectStoreKeys", {})
        self.mq_advanced_features: dict = raw_json.get("mqAdvancedFeatures", {})
        self.gateways: dict = raw_json.get("gateways", {})
        self.design_center: dict = raw_json.get("designCenter", {})
        self.partners_production: dict = raw_json.get("partnersProduction", {})
        self.partners_sandbox: dict = raw_json.get("partnersSandbox", {})
        self.trading_partners_production: dict = raw_json.get("tradingPartnersProduction", {})
        self.trading_partners_sandbox: dict = raw_json.get("tradingPartnersSandbox", {})
        self.load_balancer: dict = raw_json.get("loadBalancer", {})
        self.external_identity: bool = raw_json.get("externalIdentity", {})
        self.autoscaling: bool = raw_json.get("autoscaling", {})
        self.arm_alerts: bool = raw_json.get("armAlerts", {})
        self.apis: dict = raw_json.get("apis", {})
        self.api_monitoring: dict = raw_json.get("apiMonitoring", {})
        self.api_community_manager: dict = raw_json.get("apiCommunityManager", {})
        self.api_experience_hub: dict = raw_json.get("apiExperienceHub", {})
        self.monitoring_center: dict = raw_json.get("monitoringCenter", {})
        self.api_query: dict = raw_json.get("apiQuery", {})
        self.api_query_c360: dict = raw_json.get("apiQueryC360", {})
        self.api_governance: dict = raw_json.get("apiGovernance", {})
        self.crowd: dict = raw_json.get("crowd", {})
        self.cam: dict = raw_json.get("cam", {})
        self.exchange2: dict = raw_json.get("exchange2", {})
        self.crowd_self_service_migration: dict = raw_json.get("crowdSelfServiceMigration", {})
        self.kpi_dashboard: dict = raw_json.get("kpiDashboard", {})
        self.pcf: bool = raw_json.get("pcf", {})
        self.app_viz: bool = raw_json.get("appViz", {})
        self.runtime_fabric: bool = raw_json.get("runtimeFabric", {})
        self.anypoint_security_tokenization: dict = raw_json.get("anypointSecurityTokenization", {})
        self.anypoint_security_edge_policies: dict = raw_json.get("anypointSecurityEdgePolicies", {})
        self.runtime_fabric_cloud: dict = raw_json.get("runtimeFabricCloud", {})
        self.service_mesh: dict = raw_json.get("serviceMesh", {})
        self.flex_gateway: dict = raw_json.get("flexGateway", {})
        self.api_catalog: dict = raw_json.get("apiCatalog", {})
        self.composer: dict = raw_json.get("composer", {})
        self.mule_dx_web_ide: dict = raw_json.get("muleDxWebIde", {})
        self.messaging: dict = raw_json.get("messaging", {})
        self.worker_clouds: dict = raw_json.get("workerClouds", {})



import logging
from typing import Optional, Union

import requests

from anypoint.api.api_manager import ApiManagerApi
from anypoint.api.application import ApplicationApi
from anypoint.api.environment import EnvironmentApi
from anypoint.api.monitoring import MonitoringApi
from anypoint.api.mq import MQApi
from anypoint.api.organization import OrganizationApi
from anypoint.authentication import Authentication
from anypoint.http_client.client import HttpClient

_default_log = logging.getLogger(__name__)


class Anypoint:

    def __init__(self, authentication: Authentication,
                 base_url: str = "https://anypoint.mulesoft.com",
                 log: logging.Logger = _default_log,
                 proxies: Optional[dict] = None,
                 http_timeout: int = 30):
        self._base_url = base_url
        self._authentication = authentication

        self.organizations = OrganizationApi(self, log)
        self.applications = ApplicationApi(self, log)
        self.environments = EnvironmentApi(self, log)
        self.monitoring = MonitoringApi(self, log)
        self.api_manager = ApiManagerApi(self, log)
        self.mq = MQApi(self, log)

        self._log = log
        self._access_token: Optional[str] = None
        self._http_client = HttpClient(log, proxies, http_timeout=http_timeout)
        self._authentication.http_client = self._http_client

    def login(self):
        self._access_token = self._authentication.get_token(self._base_url)

    def me(self) -> dict:
        # TODO - Parse the response
        path = "/accounts/api/me"
        return self.request(path)

    def request(self, path: str,
                method: str = "GET",
                body: Optional[dict] = None,
                headers: Optional[dict] = None,
                parameters: Optional[dict] = None,
                return_json = True) -> Union[dict, requests.Response]:
        url = f"{self._base_url}{path}"
        if not self._access_token and not url.endswith("/accounts/login"):
            self.login()
        if headers is None:
            headers = {}
        headers.update({
            "Authorization": f"Bearer {self._access_token}"
        })

        return self._http_client.request(url, method, body, headers, parameters, return_json)



import logging
from datetime import datetime
from typing import List, Optional, TYPE_CHECKING

from anypoint.models.destination import Destination, Queue

if TYPE_CHECKING:
    from anypoint import Anypoint

# Fri, 11 Jul 2015 08:49:37 GMT
DATE_FORMAT = "%a, %d %b %Y %H:%M:%S %Z"


class MQApi:

    def __init__(self, anypoint: "Anypoint", log: logging.Logger):
        self._client = anypoint
        self._log = log

    def get_destinations(self, organization_id: str, environment_id: str, region: str) -> List[Destination]:
        #  https://anypoint.mulesoft.com/mq/admin/api/v1/organizations/{organizationId}/environments/{environmentId}/regions/{regionId}/destinations
        path = f"/mq/admin/api/v1/organizations/{organization_id}/environments/{environment_id}/regions/{region}/destinations"
        data = self._client.request(path)
        return [Destination(d, self._client, organization_id, environment_id, region) for d in data]

    def get_queues(self, organization_id: str, environment_id: str, region: str, destinations: List[str]):
        # curl -X GET "https://anypoint.mulesoft.com/mq/stats/api/v1/organizations/ORGANIZATION_ID/environments/ENVIRONMENT_ID/regions/REGION_URL/queues?destinationIds=DESTINATION_ID" \
        path = f"/mq/stats/api/v1/organizations/{organization_id}/environments/{environment_id}/regions/{region}/queues"
        params = {
            "destinationIds": ",".join(destinations) if destinations else None
        }
        return self._client.request(path, parameters=params)

    def get_queue(self,
                  organization_id: str,
                  environment_id: str,
                  region: str,
                  destination_id: str,
                  start_date: Optional[datetime] = None,
                  end_date: Optional[datetime] = None) -> Queue:
        path = f"/mq/stats/api/v1/organizations/{organization_id}/environments/{environment_id}/regions/{region}/queues/{destination_id}"
        params = {
            "startDate": start_date.strftime(DATE_FORMAT) if start_date else None,
            "endDate": end_date.strftime(DATE_FORMAT) if end_date else None,
            "period": 86400
        }
        data = self._client.request(path, parameters=params)
        return Queue(data)



import logging
from datetime import datetime
from typing import Generator, TYPE_CHECKING

from anypoint import utils
from anypoint.models.application import Application
from anypoint.models.statistics import DashboardStatistics, Statistic

if TYPE_CHECKING:
    from anypoint import Anypoint


class ApplicationApi:

    def __init__(self, anypoint: "Anypoint", log: logging.Logger):
        self._client = anypoint
        self._log = log

    def get_application_statistics(self,
                                   environment_id: str,
                                   app_domain: str,
                                   date_from: datetime,
                                   date_to: datetime) -> Generator[Statistic, None, None]:
        path = f"/cloudhub/api/applications/{app_domain}/statistics"
        headers = {"X-ANYPNT-ENV-ID": environment_id}
        params = {
            "start": f"{int(date_from.timestamp() * 1000)}",
            "end": f"{int(date_to.timestamp() * 1000)}",
            "intervalCount": f"{(date_to - date_from).total_seconds() // 60:0.0f}"
        }
        data = self._client.request(path, parameters=params, headers=headers)
        for timestamp, value in data.items():
            yield Statistic(timestamp, value)

    def get_dashboard_statistics(self,
                                 environment_id: str,
                                 app_domain: str,
                                 date_from: datetime,
                                 date_to: datetime) -> DashboardStatistics:
        path = f"/cloudhub/api/v2/applications/{app_domain}/dashboardStats"
        headers = {"X-ANYPNT-ENV-ID": environment_id}
        params = {
            "startDate": utils.date_to_str(date_from),
            "endDate": utils.date_to_str(date_to),
            "interval": "60000"
        }
        data = self._client.request(path, parameters=params, headers=headers)
        return DashboardStatistics(data)

    def get_applications(self, environment_id: str) -> Generator[Application, None, None]:
        path = f"/cloudhub/api/applications"
        headers = {"X-ANYPNT-ENV-ID": environment_id}
        data = self._client.request(path, headers=headers)
        for app in data:
            app["environment_id"] = environment_id
            yield Application(app, self)

    def get_insights(self, environment_id: str, app_domain: str):
        path = f"/api/v2/applications/{app_domain}/insight"
        headers = {"X-ANYPNT-ENV-ID": environment_id}
        return self._client.request(path, headers=headers)

    def get_application_status(self, environment_id: str, app_domain: str):
        path = f"/cloudhub/api/applications/{app_domain}/status"
        headers = {"X-ANYPNT-ENV-ID": environment_id}
        response = self._client.request(path, headers=headers, return_json=False)
        return response.text



import logging
from typing import Optional, Union

import requests
# Disable urllib warnings
from requests.adapters import HTTPAdapter
from urllib3 import Retry

requests.packages.urllib3.disable_warnings()

REQUEST_TIMEOUT = 60


class CustomRetry(Retry):
    def get_backoff_time(self):
        return self.backoff_factor


class HttpClient:

    def __init__(self, log: logging.Logger, proxies: Optional[dict] = None, http_timeout: int = REQUEST_TIMEOUT):
        self._log = log
        self._proxies = proxies
        self._session = requests.Session()
        self.http_timeout = http_timeout
        self._session.mount('https://', HTTPAdapter(max_retries=CustomRetry(
            total=3,
            backoff_factor=0.5,
            status_forcelist=[429, 500, 502, 503, 504],
            method_whitelist=["TRACE", "PUT", "DELETE", "OPTIONS", "HEAD", "GET", "POST"],
            raise_on_status=False,
        )))

    def request(self, url: str,
                method: str = "GET",
                body: Optional[dict] = None,
                headers: Optional[dict] = None,
                parameters: Optional[dict] = None,
                return_json = True
                ) -> Union[dict, requests.Response]:
        try:
            r = self._session.request(method,
                                      url,
                                      json=body,
                                      params=parameters,
                                      headers=headers,
                                      timeout=self.http_timeout,
                                      proxies=self._proxies,
                                      verify=False)
            self._log.debug(f"Received response {method} {url} {parameters if parameters else ''}: {r}")
            if r.status_code >= 400:
                error_message = f"Received a bad response: {method} {url}: {r}: {r.content}"
                raise Exception(error_message)
            # self._log.debug(f"Response: {r.json()}")
            return r.json() if return_json else r
        except Exception as e:
            self._log.exception(f"Could not make request {method} {url}: {repr(e)}")
            raise e
